%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% WN_CONNECT source v0.00 : wn_hypernyms module
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(wn_hypernyms, [
		wn_hypernyms/2,
        wn_display_hypernyms/1,
        wn_display_graph_hypernyms/1,
        wn_hypernyms/3
   ]).


:- use_module(wn_synsets).
:- use_module(wn_display_graph).

%:- op(600,xfy,'#').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% wn_hypernyms(+W_Hyponym, -List_SynSet_HyperNym)
%%% Given a word (term) "W_Hyponym" returns the list "List_SynSet_HyperNym" of its
%%% hypernym synset_IDs
%%%
%%% The word "W_Hyponym" is a term with the following syntax:
%%%                      Word[:SS_type[:Sense_num]]
%%% Where ss_type is a one character code indicating the synset type:
%%    n NOUN
%%    v VERB
%%    a ADJECTIVE
%%    s ADJECTIVE SATELLITE
%%    r ADVERB
%%
%%% and "Sense_num" specifies the sense number (meaning) of the word, within the part of
%%% speech encoded in the synset_id. "Sense_num" is a natural number: 1, 2, 3, ...

wn_hypernyms(Word:SS_type:Sense_num, List_SynSet_HyperNym) :-
        !,
        wordnet:wn_s(SynSet_ID, _, Word, SS_type, Sense_num, _),
        ((SS_type=n ; SS_type=v) ->
                hypernym_chain(SynSet_ID, List_SynSet_HyperNym)
                ;
                write(">>>> "), write(Word:SS_type:Sense_num), nl,
                wordnet:wn_g(SynSet_ID, Glos),
                write(Glos), nl,
                write("Adjectives and adverbs do not have hypernyms"),
                nl,
                fail
        ).

wn_hypernyms(Word:SS_type, List_SynSet_HyperNym) :-
        !,
        wordnet:wn_s(SynSet_ID, _, Word, SS_type, _, _),
        ((SS_type=n ; SS_type=v) ->
            hypernym_chain(SynSet_ID, List_SynSet_HyperNym)
            ;
            write(">>>> "), write(Word:SS_type), nl,
            wordnet:wn_g(SynSet_ID, Glos),
            write(Glos), nl,
            write("Adjectives and adverbs do not have hypernyms"),
            nl,
            fail
        ).

wn_hypernyms(Word, List_SynSet_HyperNym) :-
        wordnet:wn_s(SynSet_ID, _, Word, SS_type, Sense_num, _),
        ((SS_type=n ; SS_type=v) ->
            hypernym_chain(SynSet_ID, List_SynSet_HyperNym)
            ;
            write(">>>> "), write(Word:SS_type:Sense_num), nl,
            wordnet:wn_g(SynSet_ID, Glos),
            write(Glos), nl,
            write(" Adjectives and adverbs do not have hypernyms"),
            nl, nl,
            fail
        ).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  TEXTUAL REPRESENTATION OF HYPERNYMS CHAINS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% wn_display_hypernyms(+W_Hyponym)
%%% Given a word (term) "W_Hyponym" prints a textual representation of the chain of its
%%% hypernym synsets
%%%
%%% The word "W_Hyponym" is a term with the following syntax:
%%%                      Word[:SS_type[:Sense_num]]
%%% as explained in the predicate wn_hypernyms/2

wn_display_hypernyms(W_Hyponym) :-
        wn_hypernyms(W_Hyponym, List_SynSet_HyperNym),
        display_hypernym_list(List_SynSet_HyperNym).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% display_hypernym_list(+List_SynSet_HyperNym)

display_hypernym_list([]).
display_hypernym_list([SynSet_ID|List_SynSet_IDs]):-
        wn_synsets:wn_synset_components(SynSet_ID, Synset_Words),
        write(Synset_Words), nl,
        write(' >> '),
        display_hypernym_list(List_SynSet_IDs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% hypernym_chain(+SynSet_Hyponym, -List_SynSet_HyperNym)
%%% Given a synset_ID "SynSet_Hyponym" returns the list "List_SynSet_HyperNym" of its
%%% hypernym synset_IDs
%%%
hypernym_chain(SynSet_Hyponym, List_SynSet_HyperNym):-
        hypernym_chain(SynSet_Hyponym, [SynSet_Hyponym], List_SynSet_HyperNym).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% hypernym_chain(+SynSet_Hyponym, +List_SynSet_HyperNym_Acc, -List_SynSet_HyperNym)

hypernym_chain(SS_Hyp, List_SS_Hyper_Acc, List_SS_Hyper) :-
        (wordnet:wn_hyp(SS_Hyp, SS_Hyper),
        hypernym_chain(SS_Hyper, [SS_Hyper|List_SS_Hyper_Acc], List_SS_Hyper)
        ;
        \+(wordnet:wn_hyp(SS_Hyp, _SS_Hyper)),
        List_SS_Hyper = List_SS_Hyper_Acc
		).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  GRAPHICAL REPRESENTATION OF HYPERNYMS CHAINS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% wn_display_graph_hypernyms(+Word)
%%% It shows a graphic representation of all hypernyms corresponding to all the senses
%%% of the word 'Word'.
%%%
%%% A node of the graph only shows the representative word of that hypernym synset (see below).
%%%
wn_display_graph_hypernyms(Word) :-
setof(arc(X,Y),List^H^T^(list_of_representative_hypernyms(Word,List),append(H,[X,Y|T],List)),Graph), display_graph(Graph).

%%% list_of_representative_hypernyms(+Word,-List_representative_HyperNyms)
%%%
%%%
list_of_representative_hypernyms(Word,List_representative_HyperNyms) :-
        wn_hypernyms(Word, List_SynSet_HyperNyms),
        convert_synsetIDs_to_representatives(List_SynSet_HyperNyms, List_representative_HyperNyms).


%%% convert_synsetIDs_to_representatives(+List_SynSet_HyperNym, -List_representative_HyperNym)
%%% List_SynSet_HyperNym is a list of synset_IDs which are hypernyms of a given word.
%%% This predicate converts List_SynSet_HyperNym into a list of representative words.
%%%
%%% We mean by "representative word" the first word of the synset. The one with W_num=1. This is
%%% usually the most representative word of the sysntet.
%%%
convert_synsetIDs_to_representatives([],[]).
convert_synsetIDs_to_representatives([SynSet_ID|SynSet_IDs],[Word_string|Representatives]):-
        wordnet:wn_s(SynSet_ID, 1, Word, SS_type, Sense_num, _),
        word_term_to_string(Word:SS_type:Sense_num, Word_string),
        convert_synsetIDs_to_representatives(SynSet_IDs, Representatives).


%%% word_term_to_string(+Word:SS_type:Sense_num, -Word_string)
%%%
%%% Converts a word term Word:SS_type:Sense_num into a string "<Word>_<SS_type>_<Sense_num>".
%%% For instance, the word term 'psychological feature':n:1 is converted into "psychological_feature_n_1".
%%%
word_term_to_string(Word:SS_type:Sense_num, Word_string):-
    %% adapting Word, which may be a composed atom (e.g. 'psychological feature')
    atom_string(Word, S), split_string(S, " ", " ", L1),
    list_strings_to_string(L1, S1),
    string_concat(S1, "_", S2),
    string_concat(S2, SS_type, S3),
    string_concat(S3, "_", S4),
    number_string(Sense_num, SN),
    string_concat(S4, SN, Word_string).


%%% list_strings_to_string(+Lis_of_strings, +String)
%%% Concatenates the Lis_of_strings into one String.
%%%
list_strings_to_string([],"").
list_strings_to_string([Str],Str):- !.
list_strings_to_string([Str|StrLists], String) :-
        string_concat(Str, "_", S),
        list_strings_to_string(StrLists, SS),
        string_concat(S, SS, String).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  HYPERNYMS CHAINS FOR SIMILARITY MEASURES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% wn_hypernyms(+W_Hyponym, -List_SynSet_HyperNym, -Virtual_Root_ID)
%%% Given a word (term) "W_Hyponym" returns the list "List_SynSet_HyperNym" of its
%%% hypernym synset_IDs AND a virtual root.
%%%
%%% If "W_Hyponym" is a noun Virtual_Root_ID = 100000000 ('Root':n:1)
%%% If "W_Hyponym" is a verb Virtual_Root_ID = 200000000 ('Root':v:1)
%%%
%%% NOTE:
%%% This is a predicate for internal use and it will be hiden for users of the system.
%%% "W_Hyponym" must be introduced with the following syntax: "Word:SS_type:Sense_num"
%%%

wn_hypernyms(Word:SS_type:Sense_num, List_SynSet_HyperNym, Virtual_Root_ID) :-
    wn_hypernyms(Word:SS_type:Sense_num, List_SynSet_HyperNym),
    (SS_type = n ->
            Virtual_Root_ID = 100000000
            ;
            Virtual_Root_ID = 200000000
    ).






